@@ -24,13 +24,18 @@ TODO:
 """
 
 __author__ = "Sylvain Thenault"
-__revision__ = "$Id: builder.py,v 1.29 2005/04/25 14:47:06 syt Exp $"
+__revision__ = "$Id: builder.py,v 1.30 2005/05/27 11:10:25 syt Exp $"
 
 import sys
+from os.path import splitext, basename, dirname, exists, abspath
 from compiler import parse
 from inspect import isfunction, ismethod, ismethoddescriptor, isclass, \
-     isbuiltin, isdatadescriptor, getargspec
-from os.path import splitext, basename, dirname, exists, abspath
+     isbuiltin, getargspec
+try: # python 2.2 inspect module doesn't have the isdatadescriptor function
+    from inspect import isdatadescriptor
+except ImportError:
+    def isdatadescriptor(obj):
+        return False
 
 from logilab.common.fileutils import norm_read
 from logilab.common.modutils import modpath_from_file
