@@ -18,14 +18,14 @@
 possible by providing a class responsible to get astng representation
 from various source and using a cache of built modules)
 
-:version:   $Revision: 1.33 $  
+:version:   $Revision: 1.34 $  
 :author:    Sylvain Thenault
 :copyright: 2003-2005 LOGILAB S.A. (Paris, FRANCE)
 :copyright: 2003-2005 Sylvain Thenault
 :contact:   http://www.logilab.fr/ -- mailto:python-projects@logilab.org
 """
 
-__revision__ = "$Id: manager.py,v 1.33 2005/04/07 15:42:35 syt Exp $"
+__revision__ = "$Id: manager.py,v 1.34 2005/04/25 14:47:06 syt Exp $"
 __doctype__ = "restructuredtext en"
 
 import sys
@@ -38,13 +38,6 @@ from logilab.common.modutils import NoSourceFile, is_python_source, \
 from logilab.common.configuration import OptionsProviderMixIn
 from logilab.common.astng import ASTNGBuildingException
 
-def normalize_module_name(modname):
-    """normalize a module name (i.e remove trailing __init__ if any)
-    """
-    parts = modname.split('.')
-    if parts[-1] == '__init__':
-        return '.'.join(parts[:-1])
-    return modname
 
 def astng_wrapper(func, modname):
     """wrapper to give to ASTNGManager.project_from_files"""
@@ -83,7 +76,8 @@ class ASTNGManager(OptionsProviderMixIn):
         self.__dict__ = ASTNGManager.brain
         if not self.__dict__:
             OptionsProviderMixIn.__init__(self)
-            self._cache = Cache(200)
+            self._cache = None
+            self.set_cache_size(200)
 
     def set_cache_size(self, cache_size):
         """set the cache size
@@ -121,7 +115,8 @@ class ASTNGManager(OptionsProviderMixIn):
             elif fallback and modname:
                 return self.astng_from_module_name(modname)
             else:
-                raise ASTNGBuildingException('unable to get astng for file %s' % filepath)
+                raise ASTNGBuildingException('unable to get astng for file %s' %
+                                             filepath)
         self._cache[filepath] = astng
         return astng
     
@@ -143,18 +138,19 @@ class ASTNGManager(OptionsProviderMixIn):
                                  
     def astng_from_module(self, module, modname=None):
         """given an imported module, return the astng object"""
+        modname = modname or module.__name__        
         try:
             # some builtin modules don't have __file__ attribute
             filepath = module.__file__
             if is_python_source(filepath):
-                return self.astng_from_file(filepath, modname or module.__name__)
+                return self.astng_from_file(filepath, modname)
         except AttributeError:
             pass
         try:
             return self._cache[modname]
         except KeyError:
             from logilab.common.astng.builder import ASTNGBuilder
-            astng = ASTNGBuilder(self).build_from_module(module, modname)
+            astng = ASTNGBuilder(self).module_build(module, modname)
             # update caches
             self._cache[modname] = astng
             return astng
@@ -268,8 +264,8 @@ class Package:
                 if fname.endswith('.py'):
                     self.__keys.append(fname[:-3])
                     continue
-                abspath = join(self.path, fname)
-                if isdir(abspath) and exists(join(abspath, '__init__.py')):
+                fpath = join(self.path, fname)
+                if isdir(fpath) and exists(join(fpath, '__init__.py')):
                     self.__keys.append(fname)
             self.__keys.sort()
         return self.__keys[:]
@@ -318,12 +314,3 @@ class Project:
     def __repr__(self):
         return '<Project %r at %s (%s modules)>' % (self.name, id(self),
                                                     len(self.modules))
-        
-if __name__ == '__main__':
-    obj_code = ASTNGManager().astng_from_module_name(sys.argv[1])
-    print repr(obj_code)
-    print obj_code.locals.keys()
-    #print [str(c.__class__) for c in obj_code.children]
-    print obj_code.doc
-
-
