@@ -17,7 +17,7 @@
 """
 
 __author__ = "Sylvain Thenault"
-__revision__ = "$Id: raw_building.py,v 1.3 2005/02/04 16:43:07 syt Exp $"
+__revision__ = "$Id: raw_building.py,v 1.4 2005/04/25 14:47:07 syt Exp $"
 
 import sys
 from logilab.common.astng import astng
@@ -28,7 +28,7 @@ def build_module(name, doc=None):
     node.node.parent = node
     node.object = None
     node.name = name
-    node.pure_python = 0
+    node.pure_python = False
     node.parent = None
     node.globals = node.locals = {}
     return node
@@ -42,7 +42,8 @@ if sys.version_info >= (2, 4):
         """create and initialize a astng Function node"""
         args, defaults = args or [], defaults or []
         # first argument is now a list of decorators
-        func = astng.Function([], name, args, defaults, flag, doc, astng.Stmt([]))
+        func = astng.Function([], name, args, defaults, flag, doc,
+                              astng.Stmt([]))
         func.code.parent = func
         func.locals = {}
         func.object = None
@@ -74,15 +75,16 @@ def build_class(name, basenames=None, doc=None):
     return klass
 
 def build_name_assign(name, value):
-    """create and initialize a astng Assign for a name assignment"""
+    """create and initialize an astng Assign for a name assignment"""
     return astng.Assign([astng.AssName(name, 'OP_ASSIGN')], astng.Const(value))
 
 def build_attr_assign(name, value, attr='self'):
-    """create and initialize a astng Assign for an attribute assignment"""
+    """create and initialize an astng Assign for an attribute assignment"""
     return astng.Assign([astng.AssAttr(astng.Name(attr), name, 'OP_ASSIGN')],
                         astng.Const(value))
 
 def build_from_import(fromname, names):
+    """create and intialize an astng From import statement"""
     return astng.From(fromname, [(name, None) for name in names])
 
 def register_arguments(node, args):
