@@ -1,4 +1,4 @@
-# Copyright (c) 2002-2004 LOGILAB S.A. (Paris, FRANCE).
+# Copyright (c) 2002-2005 LOGILAB S.A. (Paris, FRANCE).
 # http://www.logilab.fr/ -- mailto:contact@logilab.fr
 #
 # This program is free software; you can redistribute it and/or modify it under
@@ -16,32 +16,48 @@
 """A generic function to get a database connection.
 """
 
-__revision__ = "$Id: db.py,v 1.23 2005/04/14 07:58:10 ludal Exp $"
+__revision__ = "$Id: db.py,v 1.27 2005/08/08 10:42:24 adim Exp $"
 
 import sys
 
-__all__ = ['UnknownDriver', 'set_prefered_driver', 'get_connexion',
-           'binary', 'NoAdapterFound', 'ADAPTER_DIRECTORY',
-           'get_dbapi_compliant_module', 'PREFERED_DRIVERS',
-           'PyConnection', 'PyCursor',]
+from logilab.common import deprecated_function
+
+__all__ = ['get_dbapi_compliant_module', 'get_connexion',
+           'get_connection', 'set_prefered_driver',
+           'PyConnection', 'PyCursor',
+           'UnknownDriver', 'NoAdapterFound',
+           ]
 
 class UnknownDriver(Exception):
     """raised when a unknown driver is given to get connexion"""
 
+class NoAdapterFound(Exception):
+    """Raised when no Adpater to DBAPI was found"""
+    def __init__(self, obj, objname=None, protocol='DBAPI'):
+        if objname is None:
+            objname = obj.__name__
+        Exception.__init__(self, "Could not adapt %s to protocol %s" %
+                           (objname, protocol))
+        self.adapted_obj = obj
+        self.objname = objname
+        self._protocol = protocol
+
 
 PREFERED_DRIVERS = {
     "postgres" : [ 'psycopg', 'pgdb', 'pyPgSQL.PgSQL', ],
-    "mysql" : ['MySQLdb'], # , 'pyMySQL.MySQL],
+    "mysql" : [ 'MySQLdb', ], # 'pyMySQL.MySQL, ],
+    "sqlite" : [ 'sqlite', ],
     }
 
-def _import_driver_module(driver, drivers, imported_elements = None,
-                          quiet = True):
+def _import_driver_module(driver, drivers, imported_elements=None, quiet=True):
     """Imports the first module found in 'drivers' for 'driver'
 
     :rtype: tuple
     :returns: the tuple module_object, module_name where module_object
               is the dbapi module, and modname the module's name
     """
+    if not driver in drivers:
+        raise UnknownDriver(driver)
     imported_elements = imported_elements or []
     for modname in drivers[driver]:
         try:
@@ -57,7 +73,6 @@ def _import_driver_module(driver, drivers, imported_elements = None,
         raise ImportError('Unable to import a %s module' % driver)
     return module, modname
 
-
 def set_prefered_driver(database, module, _drivers=PREFERED_DRIVERS):
     """sets the prefered driver module for database
     database is the name of the db engine (postgresql, mysql...)
@@ -69,7 +84,6 @@ def set_prefered_driver(database, module, _drivers=PREFERED_DRIVERS):
         modules = _drivers[database]
     except KeyError:
         raise UnknownDriver('Unknown database %s' % database)
-
     # Remove module from modules list, and re-insert it in first position
     try:
         modules.remove(module)
@@ -78,41 +92,10 @@ def set_prefered_driver(database, module, _drivers=PREFERED_DRIVERS):
     modules.insert(0, module)
 
 
-
-def binary(driver='postgres', drivers = PREFERED_DRIVERS):
-    """return the binary wrapper for the given driver, (None if not found)
-    """
-    module, modname = _import_driver_module(driver, drivers, ['Binary'])
-    try:
-        return module.Binary
-    except AttributeError:
-        return None
-
-
-def system_database(driver='postgres'):
-    """return the system database for the given driver"""
-    if driver == 'postgres':
-        return 'template1'
-    raise UnknownDriver('Unknown driver %s' % driver)
-
-
-
-class NoAdapterFound(Exception):
-    """Raised when no Adpater to DBAPI was found"""
-    def __init__(self, obj, objname = None, protocol = 'DBAPI'):
-        if objname is None:
-            objname = obj.__name__
-        Exception.__init__(self, "Could not adapt %s to protocol %s" %
-                           (objname, protocol))
-        self.adapted_obj = obj
-        self.objname = objname
-        self._protocol = protocol
-
-
-
-######################################################################
+## Connection and cursor wrappers ####################################
+        
 class PyConnection:
-    """ A simple connection wrapper in python (useful for profiling) """
+    """A simple connection wrapper in python (useful for profiling)"""
     def __init__(self, cnx):
         """Wraps the original connection object"""
         self._cnx = cnx
@@ -135,12 +118,11 @@ class PyConnection:
         return self._cnx.close()
 
     def __getattr__(self, attrname):
-        return getattr(self._cnx, attrname)
+        return getattr(self._cnx, attrname)    
 
-    
 
 class PyCursor:
-    """ A simple cursor wrapper in python (useful for profiling) """
+    """A simple cursor wrapper in python (useful for profiling)"""
     def __init__(self, cursor):
         self._cursor = cursor
 
@@ -173,10 +155,11 @@ class PyCursor:
     
 
 ## Adapters list #####################################################
+    
 class DBAPIAdapter:
     """Base class for all DBAPI adpaters"""
 
-    def __init__(self, native_module, pywrap = False):
+    def __init__(self, native_module, pywrap=False):
         """
         :type native_module: module
         :param native_module: the database's driver adapted module
@@ -184,7 +167,6 @@ class DBAPIAdapter:
         self._native_module = native_module
         self._pywrap = pywrap
 
-
     def connect(self, host = '', database = '', user = '', password = ''):
         """Wraps the native module connect method"""
         kwargs = {'host' : host, 'database' : database,
@@ -192,7 +174,6 @@ class DBAPIAdapter:
         cnx = self._native_module.connect(**kwargs)
         return self._wrap_if_needed(cnx)
 
-
     def _wrap_if_needed(self, cnx):
         """Wraps the connection object if self._pywrap is True, and returns it
         If false, returns the original cnx object
@@ -201,20 +182,15 @@ class DBAPIAdapter:
             return PyConnection(cnx)
         else:
             return cnx
-
     
     def __getattr__(self, attrname):
         return getattr(self._native_module, attrname)
 
-##     def adapt(cls, module):
-##         return cls(module)
-##     adapt = classmethod(adapt)
-
 
 class _PgdbAdapter(DBAPIAdapter):
     """Simple PGDB Adapter to DBAPI (pgdb modules lacks Binary() and NUMBER)
     """
-    def __init__(self, native_module, pywrap = False):
+    def __init__(self, native_module, pywrap=False):
         DBAPIAdapter.__init__(self, native_module, pywrap)
         self.NUMBER = native_module.pgdbType('int2', 'int4', 'serial',
                                              'int8', 'float4', 'float8',
@@ -258,6 +234,18 @@ class _PgsqlAdapter(DBAPIAdapter):
         return getattr(self._native_module.PgSQL, attrname)
 
 
+class _SqliteAdapter(DBAPIAdapter):
+    """Simple pyPgSQL Adapter to DBAPI
+    """
+    def __init__(self, native_module, pywrap=False):
+        DBAPIAdapter.__init__(self, native_module, pywrap)
+        self.DATETIME = native_module.TIMESTAMP
+        
+    def connect(self, host = '', database = '', user = '', password = ''):
+        """Handles sqlite connexion format"""
+        cnx = self._native_module.connect(database)
+        return self._wrap_if_needed(cnx)
+
 
 class _MySqlDBAdapter(DBAPIAdapter):
     """Simple mysql Adapter to DBAPI
@@ -267,8 +255,7 @@ class _MySqlDBAdapter(DBAPIAdapter):
         kwargs = {'host' : host, 'db' : database,
                   'user' : user, 'passwd' : password or None}
         return self._native_module.connect(**kwargs)
-        
-    
+
 
 _ADAPTERS = {
     'postgres' : { 'pgdb' : _PgdbAdapter,
@@ -276,8 +263,10 @@ _ADAPTERS = {
                    'pyPgSQL.PgSQL' : _PgsqlAdapter,
                    },
     'mysql' : { 'MySQLdb' : _MySqlDBAdapter, },
+    'sqlite' : { 'sqlite' : _SqliteAdapter, },
     }
 
+
 # _AdapterDirectory could be more generic by adding a 'protocol' parameter
 # This one would become an adpater for 'DBAPI' protocol
 class _AdapterDirectory(dict):
@@ -291,17 +280,15 @@ class _AdapterDirectory(dict):
             
         # XXX Should we have a list of adapters ?
         driver_dict[modname] = adapter
-
     
     def adapt(self, database, prefered_drivers = None, pywrap = False):
         """Returns an dbapi-compliant object based for database"""
         prefered_drivers = prefered_drivers or PREFERED_DRIVERS
         module, modname = _import_driver_module(database, prefered_drivers)
         try:
-            return self[database][modname](module, pywrap = pywrap)
+            return self[database][modname](module, pywrap=pywrap)
         except KeyError:
-            raise NoAdapterFound(obj = module)
-        
+            raise NoAdapterFound(obj=module)        
 
     def get_adapter(self, database, modname):
         try:
@@ -309,27 +296,132 @@ class _AdapterDirectory(dict):
         except KeyError:
             raise NoAdapterFound(None, modname)
 
-
-
 ADAPTER_DIRECTORY = _AdapterDirectory(_ADAPTERS)
 del _AdapterDirectory
 
 
+## Helpers for DBMS specific Advanced functionalities ################
+
+class _GenericAdvFuncHelper:
+    """Generic helper, trying to provide generic way to implement
+    specific functionnalities from others DBMS
+
+    An exception is raised when the functionality is not emulatable
+    """
+    
+    def support_users(self):
+        """return True if the DBMS support users (this is usually
+        not true for in memory DBMS)
+        """
+        return True
+    
+    def support_groups(self):
+        """return True if the DBMS support groups"""
+        return True
+
+    def system_database(self):
+        """return the system database for the given driver"""
+        raise Exception('not supported by this DBMS')
+
+    def sql_current_date(self):
+        return 'CURRENT_DATE'
+    
+    def sql_current_time(self):
+        return 'CURRENT_TIME'
+    
+    def sql_current_timestamp(self):
+        return 'CURRENT_TIMESTAMP'
+    
+    def sql_create_sequence(self, seq_name):
+        return '''CREATE TABLE %s (last INTEGER);
+INSERT INTO %s VALUES (0);''' % (seq_name, seq_name)
+    
+    def sql_drop_sequence(self, seq_name):
+        return 'DROP TABLE %s;' % seq_name
+    
+    def sqls_increment_sequence(self, seq_name):
+        return ('UPDATE %s SET last=last+1;' % seq_name,
+                'SELECT last FROM %s;' % seq_name)
+
+    def increment_sequence(self, cursor, seq_name):
+        for sql in self.sqls_increment_sequence(seq_name):
+            cursor.execute(sql)
+        return cursor.fetchone()[0]
+
+    
+class _PGAdvFuncHelper(_GenericAdvFuncHelper):
+    """Postgres helper, taking advantage of postgres SEQUENCE support
+    """
+    
+    def system_database(self):
+        """return the system database for the given driver"""
+        return 'template1'
+
+    def sql_create_sequence(self, seq_name):
+        return 'CREATE SEQUENCE %s;' % seq_name
+    
+    def sql_drop_sequence(self, seq_name):
+        return 'DROP SEQUENCE %s;' % seq_name
+    
+    def sqls_increment_sequence(self, seq_name):
+        return ("SELECT nextval('%s');" % seq_name,)
+
+
+class _SqliteAdvFuncHelper(_GenericAdvFuncHelper):
+    """Generic helper, trying to provide generic way to implement
+    specific functionnalities from others DBMS
+
+    An exception is raised when the functionality is not emulatable
+    """
+    
+    def support_users(self):
+        """return True if the DBMS support users (this is usually
+        not true for in memory DBMS)
+        """
+        return False
+    
+    def support_groups(self):
+        """return True if the DBMS support groups"""
+        return False
+    
+    def sql_current_date(self):
+        return "DATE('now')"
+    
+    def sql_current_time(self):
+        return "TIME('now')"
+    
+    def sql_current_timestamp(self):
+        return "DATETIME('now')"
+    
+    
+ADV_FUNC_HELPER_DIRECTORY = {'postgres': _PGAdvFuncHelper(),
+                             'sqlite': _SqliteAdvFuncHelper(),
+                             None: _GenericAdvFuncHelper()}
+
+
+## Main functions ####################################################
+    
+def get_adv_func_helper(driver):
+    """returns an advanced function helper for the given driver"""
+    return ADV_FUNC_HELPER_DIRECTORY.get(driver,
+                                         ADV_FUNC_HELPER_DIRECTORY[None])
+    
 def get_dbapi_compliant_module(driver, prefered_drivers = None, quiet = False,
                                pywrap = False):
-    """Returns a fully dbapi compliant module"""
+    """returns a fully dbapi compliant module"""
     try:
-        return ADAPTER_DIRECTORY.adapt(driver, prefered_drivers, pywrap = pywrap)
+        mod = ADAPTER_DIRECTORY.adapt(driver, prefered_drivers, pywrap = pywrap)
     except NoAdapterFound, err:
         if not quiet:
             msg = 'No Adapter found for %s, returning native module' % err.objname
             print >> sys.stderr, msg
-        return err.adapted_obj
+        mod = err.adapted_obj
+    mod.adv_func_helper = get_adv_func_helper(driver)
+    return mod
 
-
-def get_connexion(driver='postgres', host='', database='', user='',
-                  password='', quiet = False, drivers = PREFERED_DRIVERS,
-                  pywrap = False):
+def get_connection(driver='postgres', host='', database='', user='',
+                  password='', quiet=False, drivers=PREFERED_DRIVERS,
+                  pywrap=False):
     """return a db connexion according to given arguments"""
     module, modname = _import_driver_module(driver, drivers, ['connect'])
     try:
@@ -343,64 +435,4 @@ def get_connexion(driver='postgres', host='', database='', user='',
         adapted_module = adapter(module, pywrap)
     return adapted_module.connect(host, database, user, password)
 
-
-def sql_repr( type, val ):
-    if type=='s':
-        return "'%s'" % (val,)
-    else:
-        return val
-
-class BaseTable:
-    # table_name = "default"
-    # supported types are s/i/d
-    # table_fields = ( ('first_field','s'), )
-    # primary_key = 'first_field'
-
-    def __init__(self, table_name, table_fields, primary_key=None ):
-        if primary_key is None:
-            self._primary_key = table_fields[0][0]
-        else:
-            self._primary_key = primary_key
-
-        self._table_fields = table_fields
-        self._table_name = table_name
-        info = {
-            'key' : self._primary_key,
-            'table' : self._table_name,
-            'columns' : ",".join( [ f for f,t in self._table_fields ] ),
-            'values' : ",".join( [ sql_repr( t, "%%(%s)s" % f ) for f,t in self._table_fields ] ),
-            'updates' : ",".join( [ "%s=%s" % ( f, sql_repr( t, "%%(%s)s" % f) ) for f,t in self._table_fields] ),
-            }
-        self._insert_stmt = "INSERT into %(table)s (%(columns)s) VALUES (%(values)s) WHERE %(key)s=%%(key)s" % info
-        self._update_stmt = "UPDATE %(table)s SET (%(updates)s) VALUES WHERE %(key)s=%%(key)s" % info
-        self._select_stmt = "SELECT %(columns)s FROM %(table)s WHERE %(key)s=%%(key)s" % info
-        self._delete_stmt = "DELETE FROM %(table)s WHERE %(key)s=%%(key)s" % info
-
-        for k,t in table_fields:
-            if hasattr(self,k):
-                raise ValueError("Cannot use %s as a table field" % k)
-            setattr(self,k,None)
-
-
-    def as_dict( self ):
-        d = {}
-        for k,t in self._table_fields:
-            d[k] = getattr(self,k)
-        return d
-
-    def select( self, curs ):
-        d = { 'key' : getattr(self,self._primary_key) }
-        curs.execute( self._select_stmt % d )
-        rows = curs.fetchall()
-        if len(rows)!=1:
-            raise ValueError("Select: ambiguous query returned %d rows" % len(rows) )
-        for (f,t),v in zip(self._table_fields,rows[0]):
-            setattr(self,f,v)
-
-    def update( self, curs ):
-        d = self.as_dict()
-        curs.execute( self._update_stmt % d )
-
-    def delete( self, curs ):
-        d = { 'key' : getattr(self,self._primary_key) }
-        
+get_connexion = deprecated_function(get_connection)
