@@ -1,30 +1,50 @@
 """Table management module
 """
 
-__revision__ = '$Id: table.py,v 1.7 2005/02/15 14:55:38 adim Exp $'
+__revision__ = '$Id: table.py,v 1.15 2005/08/17 14:48:36 alf Exp $'
 
-from logilab.common.compat import enumerate, sum
+from warnings import warn
 
+from logilab.common.compat import enumerate, sum, set
 
-class Table:
+class Table(object):
     """Table defines a data table with column and row names.
     inv:
         len(self.data) <= len(self.row_names)
         forall(self.data, lambda x: len(x) <= len(self.col_names))
     """
 
-    def __init__(self):
+    def __init__(self, default_value=0, col_names=None, row_names=None):
         self.col_names = []
         self.row_names = []
         self.data = []
+        self.default_value = default_value
+        if col_names:
+            self.create_columns(col_names)
+        if row_names:
+            self.create_rows(row_names)
+        
+    def _next_row_name(self):
+        return 'row%s' % len(self.row_names)
 
+    def __iter__(self):
+        return iter(self.data)
+    
+    def __eq__(self, other):
+        if other is not None:
+            return self.data == other.data
+        return False
+
+    def __ne__(self, other):
+        return not self == other
+    
     ## Rows / Columns creation #################################################
     def create_rows(self, row_names):
         """Appends row_names to the list of existing rows
         """
         self.row_names.extend(row_names)
         for row_name in row_names:
-            self.data.append([0]*len(self.col_names))
+            self.data.append([self.default_value]*len(self.col_names))
         
     def create_columns(self, col_names):
         """Appends col_names to the list of existing columns
@@ -32,11 +52,12 @@ class Table:
         for col_name in col_names:
             self.create_column(col_name)
 
-    def create_row(self, row_name):
+    def create_row(self, row_name=None):
         """Creates a rowname to the row_names list
         """
+        row_name = row_name or self._next_row_name()
         self.row_names.append(row_name)
-        self.data.append([0]*len(self.col_names))
+        self.data.append([self.default_value]*len(self.col_names))
 
     
     def create_column(self, col_name):
@@ -44,7 +65,7 @@ class Table:
         """
         self.col_names.append(col_name)
         for row in self.data:
-            row.append(0)
+            row.append(self.default_value)
 
     ## Sort by column ##########################################################
     def sort_by_column_id(self, col_id, method = 'asc'):
@@ -54,7 +75,7 @@ class Table:
             col_index = self.col_names.index(col_id)
             self.sort_by_column_index(col_index, method)
         except ValueError:
-            raise KeyError, "Col (%s) not found in table"%(col_id)
+            raise KeyError("Col (%s) not found in table"%(col_id))
         
 
     def sort_by_column_index(self, col_index, method = 'asc'):
@@ -77,32 +98,60 @@ class Table:
             self.data.append(row)
             self.row_names.append(row_name)
 
+    def groupby(self, colname, *others):
+        """builds indexes of data
+        :returns: nested dictionnaries pointing to actual rows
+        """
+        groups = {}
+        colnames = (colname,) + others
+        col_indexes = [self.col_names.index(col_id) for col_id in colnames]
+        for row in self.data:
+            ptr = groups
+            for col_index in col_indexes[:-1]:
+                ptr = ptr.setdefault(row[col_index], {})
+            ptr = ptr.setdefault(row[col_indexes[-1]], [])
+            ptr.append(tuple(row))
+        return groups
+
+    def select(self, colname, value):
+        grouped = self.groupby(colname)
+        try:
+            return grouped[value]
+        except KeyError:
+            return []
+
+    def remove(self, colname, value):
+        col_index = self.col_names.index(colname)
+        for row in self.data[:]:
+            if row[col_index] == value:
+                self.data.remove(row)
+        
     
     ## The 'setter' part #######################################################
     def set_cell(self, row_index, col_index, data):
-        """Sets value of cell 'row_indew', 'col_index' to data
+        """sets value of cell 'row_indew', 'col_index' to data
         """
         self.data[row_index][col_index] = data
         
 
     def set_cell_by_ids(self, row_id, col_id, data):
-        """Sets value of cell mapped by row_id and col_id to data
+        """sets value of cell mapped by row_id and col_id to data
         Raises a KeyError if row_id or col_id are not found in the table
         """
         try:
             row_index = self.row_names.index(row_id)
         except ValueError:
-            raise KeyError, "Row (%s) not found in table"%(row_id)
+            raise KeyError("Row (%s) not found in table"%(row_id))
         else:
             try:
                 col_index = self.col_names.index(col_id)
                 self.data[row_index][col_index] = data
             except ValueError:
-                raise KeyError, "Column (%s) not found in table"%(col_id)
+                raise KeyError("Column (%s) not found in table"%(col_id))
     
     
     def set_row(self, row_index, row_data):
-        """Sets the 'row_index' row
+        """sets the 'row_index' row
         pre:
             type(row_data) == types.ListType
             len(row_data) == len(self.col_names)
@@ -111,7 +160,7 @@ class Table:
 
         
     def set_row_by_id(self, row_id, row_data):
-        """Sets the 'row_id' column
+        """sets the 'row_id' column
         pre:
             type(row_data) == types.ListType
             len(row_data) == len(self.row_names)
@@ -121,20 +170,21 @@ class Table:
             row_index = self.row_names.index(row_id)
             self.set_row(row_index, row_data)
         except ValueError:
-            raise KeyError, 'Row (%s) not found in table'%(row_id)
+            raise KeyError('Row (%s) not found in table'%(row_id))
         
 
-    def append_row(self, row_data, row_name):
+    def append_row(self, row_data, row_name=None):
         """Appends a row to the table
         pre:
             type(row_data) == types.ListType
             len(row_data) == len(self.col_names)
         """
+        row_name = row_name or self._next_row_name()
         self.row_names.append(row_name)
         self.data.append(row_data)
         
 
-    def insert_row(self, index, row_data, row_name):
+    def insert_row(self, index, row_data, row_name=None):
         """Appends row_data before 'index' in the table. To make 'insert'
         behave like 'list.insert', inserting in an out of range index will
         insert row_data to the end of the list
@@ -142,6 +192,7 @@ class Table:
             type(row_data) == types.ListType
             len(row_data) == len(self.col_names)
         """
+        row_name = row_name or self._next_row_name()
         self.row_names.insert(index, row_name)
         self.data.insert(index, row_data)
     
@@ -162,11 +213,11 @@ class Table:
             row_index = self.row_names.index(row_id)
             self.delete_row(row_index)
         except ValueError:
-            raise KeyError, 'Row (%s) not found in table'%(row_id)
+            raise KeyError('Row (%s) not found in table'%(row_id))
     
 
     def set_column(self, col_index, col_data):
-        """Sets the 'col_index' column
+        """sets the 'col_index' column
         pre:
             type(col_data) == types.ListType
             len(col_data) == len(self.row_names)
@@ -177,7 +228,7 @@ class Table:
 
 
     def set_column_by_id(self, col_id, col_data):
-        """Sets the 'col_id' column
+        """sets the 'col_id' column
         pre:
             type(col_data) == types.ListType
             len(col_data) == len(self.col_names)
@@ -187,7 +238,7 @@ class Table:
             col_index = self.col_names.index(col_id)
             self.set_column(col_index, col_data)
         except ValueError:
-            raise KeyError, 'Column (%s) not found in table'%(col_id)
+            raise KeyError('Column (%s) not found in table'%(col_id))
         
 
     def append_column(self, col_data, col_name):
@@ -230,52 +281,75 @@ class Table:
             col_index = self.col_names.index(col_id)
             self.delete_column(col_index)
         except ValueError:
-            raise KeyError, 'Column (%s) not found in table'%(col_id)
+            raise KeyError('Column (%s) not found in table'%(col_id))
 
     
     ## The 'getter' part #######################################################
+
     def __getitem__(self, row_index):
-        """Provided for convenience
-        """
+        """provided for convenience"""
         return self.data[row_index]
 
-    
     def get_dimensions(self):
         """Returns a tuple which represents the table's size
         """
         return len(self.row_names), len(self.col_names)
         
-
     def get_element(self, row_index, col_index):
         """Returns the element at [row_index][col_index]
         """
+        warn('Table.get_element() is deprecated, use Table.get_cell instead',
+             DeprecationWarning)
+        return self.data[row_index][col_index]
+
+    def get_cell(self, row_index, col_index):
+        return self.data[row_index][col_index]
+        
+    def get_cell_by_ids(self, row_id, col_id):
+        """Returns the element at [row_id][col_id]
+        """
+        try:
+            row_index = self.row_names.index(row_id)
+        except ValueError:
+            raise KeyError("Row (%s) not found in table"%(row_id))
+        else:
+            try:
+                col_index = self.col_names.index(col_id)
+            except ValueError:
+                raise KeyError("Column (%s) not found in table"%(col_id))
         return self.data[row_index][col_index]
 
-    
     def get_row(self, row_index):
         """Returns the 'row_index' row
         """
         return self.data[row_index]
 
-
     def get_row_by_id(self, row_id):
         """Returns the 'row_id' row
         """
-        row_index = self.row_names.index(row_id)
+        try:
+            row_index = self.row_names.index(row_id)
+        except ValueError:
+            raise KeyError("Row (%s) not found in table"%(row_id))
         return self.get_row(row_index)
 
-    
-    def get_column(self, col_index):
+    def get_column(self, col_index, distinct=False):
         """Returns the 'col_index' col
         """
-        return [row[col_index] for row in self.data]
-
+        col = [row[col_index] for row in self.data]
+        if distinct:
+            return set(col)
+        else:
+            return col
     
-    def get_column_by_id(self, col_id):
+    def get_column_by_id(self, col_id, distinct=False):
         """Returns the 'col_id' col
         """
-        col_index = self.col_names.index(col_id)
-        return self.get_column(col_index)
+        try:
+            col_index = self.col_names.index(col_id)
+        except ValueError:
+            raise KeyError("Column (%s) not found in table"%(col_id))
+        return self.get_column(col_index, distinct)
     
 
     def get_rows(self):
@@ -384,7 +458,7 @@ class TableStyle:
        
     # XXX FIXME : params order should be reversed for all set() methods
     def set_size(self, value, col_id):
-        """Sets the size of the specified col_id to value
+        """sets the size of the specified col_id to value
         """
         self.size[col_id] = value
 
@@ -402,7 +476,7 @@ class TableStyle:
 
 
     def set_alignment(self, value, col_id):
-        """Sets the alignment of the specified col_id to value
+        """sets the alignment of the specified col_id to value
         """
         self.alignment[col_id] = value
 
@@ -421,7 +495,7 @@ class TableStyle:
 
 
     def set_unit(self, value, col_id):
-        """Sets the unit of the specified col_id to value
+        """sets the unit of the specified col_id to value
         """
         self.units[col_id] = value
 
@@ -725,13 +799,13 @@ class TableWriter:
         
 
     def set_style(self, style):
-        """Sets the table's associated style
+        """sets the table's associated style
         """
         self.style = style
 
 
     def set_renderer(self, renderer):
-        """Sets the way to render cell
+        """sets the way to render cell
         """
         self.renderer = renderer
     
@@ -745,7 +819,7 @@ class TableWriter:
     def write_table(self, title = ""):
         """Writes the table
         """
-        raise NotImplementedError, "write_table must be implemented !"
+        raise NotImplementedError("write_table must be implemented !")
         
 
 
