@@ -2,10 +2,11 @@
 unit tests for module logilab.common.db
 
 """
-__revision__ = "$Id: unittest_db.py,v 1.7 2005/02/15 14:56:31 adim Exp $"
+__revision__ = "$Id: unittest_db.py,v 1.9 2005/08/08 10:42:25 adim Exp $"
 
 import unittest
 from logilab.common.db import *
+from logilab.common.db import PREFERED_DRIVERS, _GenericAdvFuncHelper, _PGAdvFuncHelper
 
 class PreferedDriverTC(unittest.TestCase):
     def setUp(self):
@@ -50,36 +51,36 @@ class getCnxTC(unittest.TestCase):
     def testPsyco(self):
         set_prefered_driver('postgres', 'psycopg')
         try:
-            cnx = get_connexion('postgres',
-                                self.host, self.db, self.user, self.passwd,
-                                quiet=1)
+            cnx = get_connection('postgres',
+                                 self.host, self.db, self.user, self.passwd,
+                                 quiet=1)
         except ImportError:
             self.fail('python-psycopg is not installed')
 
     def testPgdb(self):
         set_prefered_driver('postgres', 'pgdb')
         try:
-            cnx = get_connexion('postgres',
-                                self.host, self.db, self.user, self.passwd,
-                                quiet=1)
+            cnx = get_connection('postgres',
+                                 self.host, self.db, self.user, self.passwd,
+                                 quiet=1)
         except ImportError:
             self.fail('python-pgsql is not installed')
 
     def testPgsql(self):
         set_prefered_driver('postgres', 'pyPgSQL.PgSQL')
         try:
-            cnx = get_connexion('postgres',
-                                self.host, self.db, self.user, self.passwd,
-                                quiet=1)
+            cnx = get_connection('postgres',
+                                 self.host, self.db, self.user, self.passwd,
+                                 quiet=1)
         except ImportError:
             self.fail('python-pygresql is not installed')
 
     def testMysql(self):
         set_prefered_driver('mysql', 'MySQLdb')
         try:
-            cnx = get_connexion('mysql',
-                                self.host, self.db, self.user, self.passwd,
-                                quiet=1)
+            cnx = get_connection('mysql',
+                                 self.host, self.db, self.user, self.passwd,
+                                 quiet=1)
         except ImportError:
             self.fail('python-mysqldb is not installed')
         except Exception, ex:
@@ -88,6 +89,29 @@ class getCnxTC(unittest.TestCase):
             if not (isinstance(ex, MySQLdb.OperationalError) and ex.args[0] == 2003):
                 raise
 
+    def test_connection_wrap(self):
+        """Tests the connection wrapping"""
+        cnx = get_connection('postgres',
+                             self.host, self.db, self.user, self.passwd,
+                             quiet=1)
+        self.failIf(isinstance(cnx, PyConnection),
+                    'cnx should *not* be a PyConnection instance')
+        cnx = get_connection('postgres',
+                             self.host, self.db, self.user, self.passwd,
+                             quiet=1, pywrap = True)
+        self.failUnless(isinstance(cnx, PyConnection),
+                        'cnx should be a PyConnection instance')
+        
+
+    def test_cursor_wrap(self):
+        """Tests cursor wrapping"""
+        cnx = get_connection('postgres',
+                             self.host, self.db, self.user, self.passwd,
+                             quiet=1, pywrap = True)
+        cursor = cnx.cursor()
+        self.failUnless(isinstance(cursor, PyCursor),
+                        'cnx should be a PyCursor instance')
+
 
 class DBAPIAdaptersTC(unittest.TestCase):
     """Tests DbApi adapters management"""
@@ -104,6 +128,9 @@ class DBAPIAdaptersTC(unittest.TestCase):
         """Reset PREFERED_DRIVERS as it was"""
         PREFERED_DRIVERS['postgres'] = self.old_drivers
 
+    def test_raise(self):
+        self.assertRaises(UnknownDriver, get_dbapi_compliant_module, 'pougloup')
+        
     def test_pgdb_types(self):
         """Tests that NUMBER really wraps all number types"""
         set_prefered_driver('postgres', 'pgdb')        
@@ -115,7 +142,6 @@ class DBAPIAdaptersTC(unittest.TestCase):
             self.assertEquals(num_type, module.NUMBER)
         self.assertNotEquals('char', module.NUMBER)
 
-
     def test_pypgsql_getattr(self):
         """Tests the getattr() delegation for pyPgSQL"""
         set_prefered_driver('postgres', 'pyPgSQL.PgSQL')
@@ -123,32 +149,14 @@ class DBAPIAdaptersTC(unittest.TestCase):
         try:
             binary = module.BINARY
         except AttributeError, err:
-            self.fail(str(err))
-        
+            self.fail(str(err))        
 
-    def test_connection_wrap(self):
-        """Tests the connection wrapping"""
-        cnx = get_connexion('postgres',
-                            self.host, self.db, self.user, self.passwd,
-                            quiet=1)
-        self.failIf(isinstance(cnx, PyConnection),
-                    'cnx should *not* be a PyConnection instance')
-        cnx = get_connexion('postgres',
-                            self.host, self.db, self.user, self.passwd,
-                            quiet=1, pywrap = True)
-        self.failUnless(isinstance(cnx, PyConnection),
-                        'cnx should be a PyConnection instance')
-        
+    def test_adv_func_helper(self):
+        module = get_dbapi_compliant_module('postgres')
+        self.failUnless(isinstance(module.adv_func_helper, _PGAdvFuncHelper))
+        module = get_dbapi_compliant_module('sqlite')
+        self.failUnless(isinstance(module.adv_func_helper, _GenericAdvFuncHelper))
 
-    def test_cursor_wrap(self):
-        """Tests cursor wrapping"""
-        cnx = get_connexion('postgres',
-                            self.host, self.db, self.user, self.passwd,
-                            quiet=1, pywrap = True)
-        cursor = cnx.cursor()
-        self.failUnless(isinstance(cursor, PyCursor),
-                        'cnx should be a PyCursor instance')
-        
 
 if __name__ == '__main__':
     unittest.main()
