@@ -2,12 +2,13 @@
 Unittests for table management
 """
 
-__revision__ = '$Id: unittest_table.py,v 1.5 2005/02/15 14:55:39 adim Exp $'
+__revision__ = '$Id: unittest_table.py,v 1.11 2005/08/19 16:16:23 nico Exp $'
 
 import unittest
-import sys 
+import sys
 from logilab.common.table import Table, TableStyleSheet, DocbookTableWriter, \
      DocbookRenderer, TableStyle, TableWriter, TableCellRenderer
+from logilab.common.compat import set
 from cStringIO import StringIO
 
 class TableTC(unittest.TestCase):
@@ -24,10 +25,14 @@ class TableTC(unittest.TestCase):
 
     def test_indexation(self):
         """we should be able to use [] to access rows"""
-        row0 = self.table.data[0]
-        row1 = self.table.data[1]
-        self.assert_(self.table[0] is row0)
-        self.assert_(self.table[1] is row1)
+        self.assert_(self.table[0] is self.table.data[0])
+        self.assert_(self.table[1] is self.table.data[1])
+
+    def test_iterable(self):
+        """test iter(table)"""
+        it = iter(self.table)
+        self.assert_(it.next() is self.table.data[0])
+        self.assert_(it.next() is self.table.data[1])
 
     def test_get_rows(self):
         """tests Table.get_rows()"""
@@ -35,6 +40,17 @@ class TableTC(unittest.TestCase):
         self.table.insert_column(1, range(3), 'supp')
         self.assertEquals(self.table.get_rows(), [[0, 0, 0], [0, 1, 0], [0, 2, 0]])
         
+    def test_get_cells(self):
+        self.table.insert_column(1, range(3), 'supp')
+        self.assertEquals(self.table.get_cell(0, 1), 0)
+        self.assertEquals(self.table.get_cell(1, 1), 1)
+        self.assertEquals(self.table.get_cell(2, 1), 2)
+        self.assertEquals(self.table.get_cell_by_ids('row1', 'supp'), 0)
+        self.assertEquals(self.table.get_cell_by_ids('row2', 'supp'), 1)
+        self.assertEquals(self.table.get_cell_by_ids('row3', 'supp'), 2)
+        self.assertRaises(KeyError, self.table.get_cell_by_ids, 'row1', 'foo')
+        self.assertRaises(KeyError, self.table.get_cell_by_ids, 'foo', 'bar')
+        
     def test_dimensions(self):
         """tests table dimensions"""
         self.assertEquals(self.table.get_dimensions(), (3, 2))
@@ -45,29 +61,29 @@ class TableTC(unittest.TestCase):
         """Tests that table.set_column() works fine.
         """
         self.table.set_column(0, range(3))
-        self.assertEquals(self.table.get_element(0,0), 0)
-        self.assertEquals(self.table.get_element(1,0), 1)
-        self.assertEquals(self.table.get_element(2,0), 2)
+        self.assertEquals(self.table.get_cell(0,0), 0)
+        self.assertEquals(self.table.get_cell(1,0), 1)
+        self.assertEquals(self.table.get_cell(2,0), 2)
 
     def test_set_column_by_id(self):
         """Tests that table.set_column_by_id() works fine.
         """
         self.table.set_column_by_id('col1', range(3))
-        self.assertEquals(self.table.get_element(0,0), 0)
-        self.assertEquals(self.table.get_element(1,0), 1)
-        self.assertEquals(self.table.get_element(2,0), 2)
+        self.assertEquals(self.table.get_cell(0,0), 0)
+        self.assertEquals(self.table.get_cell(1,0), 1)
+        self.assertEquals(self.table.get_cell(2,0), 2)
         self.assertRaises(KeyError, self.table.set_column_by_id, 'col123', range(3))
 
     def test_cells_ids(self):
         """tests that we can access cells by giving row/col ids"""
         self.assertRaises(KeyError, self.table.set_cell_by_ids, 'row12', 'col1', 12)
         self.assertRaises(KeyError, self.table.set_cell_by_ids, 'row1', 'col12', 12)
-        self.assertEquals(self.table.get_element(0,0), 0)
+        self.assertEquals(self.table.get_cell(0,0), 0)
         self.table.set_cell_by_ids('row1', 'col1', 'DATA')
-        self.assertEquals(self.table.get_element(0,0), 'DATA')
+        self.assertEquals(self.table.get_cell(0,0), 'DATA')
         self.assertRaises(KeyError, self.table.set_row_by_id, 'row12', [])
         self.table.set_row_by_id('row1', ['1.0', '1.1'])
-        self.assertEquals(self.table.get_element(0,0), '1.0')
+        self.assertEquals(self.table.get_cell(0,0), '1.0')
 
     def test_insert_row(self):
         """tests a row insertion"""
@@ -78,7 +94,7 @@ class TableTC(unittest.TestCase):
         self.table.delete_row_by_id('tmprow')
         self.assertRaises(KeyError, self.table.delete_row_by_id, 'tmprow')
         self.assertEquals(self.table.get_row(1), [0, 0])
-        self.assertRaises(ValueError, self.table.get_row_by_id, 'tmprow')
+        self.assertRaises(KeyError, self.table.get_row_by_id, 'tmprow')
         
     def test_get_column(self):
         """Tests that table.get_column() works fine.
@@ -88,6 +104,13 @@ class TableTC(unittest.TestCase):
         self.assertEquals(self.table.get_column(1), [12,0,13])
         self.assertEquals(self.table.get_column_by_id('col2'), [12,0,13])
 
+    def test_get_column_distinct(self):
+        """tests get_column(distinct=True)"""
+        self.table.set_cell(0, 1, 12)
+        self.table.set_cell(2, 1, 12)
+        self.assertEquals(self.table.get_column(1, distinct=True), set([12,0]))
+        self.assertEquals(self.table.get_column_by_id('col2', distinct=True), set([12,0]))
+
     def test_get_columns(self):
         """Tests if table.get_columns() works fine.
         """
@@ -150,7 +173,62 @@ class TableTC(unittest.TestCase):
         """only tests pprint doesn't raise an exception"""
         self.table.pprint()
         str(self.table)
-        
+
+
+class GroupByTC(unittest.TestCase):
+    """specific test suite for groupby()"""
+    def setUp(self):
+        t = Table()
+        t.create_columns(['date', 'res', 'task', 'usage'])
+        t.append_row(['date1', 'ing1', 'task1', 0.3])
+        t.append_row(['date1', 'ing2', 'task2', 0.3])
+        t.append_row(['date2', 'ing3', 'task3', 0.3])
+        t.append_row(['date3', 'ing4', 'task2', 0.3])
+        t.append_row(['date1', 'ing1', 'task3', 0.3])
+        t.append_row(['date3', 'ing1', 'task3', 0.3])        
+        self.table = t
+
+    
+    def test_single_groupby(self):
+        """tests groupby() on several columns"""
+        grouped = self.table.groupby('date')
+        self.assertEquals(len(grouped), 3)
+        self.assertEquals(len(grouped['date1']), 3)
+        self.assertEquals(len(grouped['date2']), 1)
+        self.assertEquals(len(grouped['date3']), 2)
+        self.assertEquals(grouped['date1'], [
+            ('date1', 'ing1', 'task1', 0.3),
+            ('date1', 'ing2', 'task2', 0.3),
+            ('date1', 'ing1', 'task3', 0.3),
+            ])
+        self.assertEquals(grouped['date2'], [('date2', 'ing3', 'task3', 0.3)])
+        self.assertEquals(grouped['date3'], [
+            ('date3', 'ing4', 'task2', 0.3),
+            ('date3', 'ing1', 'task3', 0.3),
+            ])
+
+    def test_multiple_groupby(self):
+        """tests groupby() on several columns"""
+        grouped = self.table.groupby('date', 'task')
+        self.assertEquals(len(grouped), 3)
+        self.assertEquals(len(grouped['date1']), 3)
+        self.assertEquals(len(grouped['date2']), 1)
+        self.assertEquals(len(grouped['date3']), 2)
+        self.assertEquals(grouped['date1']['task1'], [('date1', 'ing1', 'task1', 0.3)])
+        self.assertEquals(grouped['date2']['task3'], [('date2', 'ing3', 'task3', 0.3)])
+        self.assertEquals(grouped['date3']['task2'], [('date3', 'ing4', 'task2', 0.3)])
+        date3 = grouped['date3']
+        self.assertRaises(KeyError, date3.__getitem__, 'task1')
+
+
+    def test_select(self):
+        """tests Table.select() method"""
+        rows = self.table.select('date', 'date1')
+        self.assertEquals(rows, [
+            ('date1', 'ing1', 'task1', 0.3),
+            ('date1', 'ing2', 'task2', 0.3),
+            ('date1', 'ing1', 'task3', 0.3),
+            ])
 
 class TableStyleSheetTC(unittest.TestCase):
     """The Stylesheet test case
@@ -196,7 +274,7 @@ class TableStyleSheetTC(unittest.TestCase):
         self.table.set_row(0, [10,20,0])
         self.stylesheet.add_rowavg_rule((0,2), 0, 0, 1)
         self.table.apply_stylesheet(self.stylesheet)
-        val = self.table.get_element(0,2)
+        val = self.table.get_cell(0,2)
         self.assert_(int(val) == 15)
         
 
@@ -206,7 +284,7 @@ class TableStyleSheetTC(unittest.TestCase):
         self.table.set_row(0, [10,20,0])
         self.stylesheet.add_rowsum_rule((0,2), 0, 0, 1)
         self.table.apply_stylesheet(self.stylesheet)
-        val = self.table.get_element(0,2)
+        val = self.table.get_cell(0,2)
         self.assert_(val == 30)
         
 
@@ -218,7 +296,7 @@ class TableStyleSheetTC(unittest.TestCase):
         self.table.create_row('row3')
         self.stylesheet.add_colavg_rule((2,0), 0, 0, 1)
         self.table.apply_stylesheet(self.stylesheet)
-        val = self.table.get_element(2,0)
+        val = self.table.get_cell(2,0)
         self.assert_(int(val) == 11)
         
 
@@ -230,9 +308,11 @@ class TableStyleSheetTC(unittest.TestCase):
         self.table.create_row('row3')
         self.stylesheet.add_colsum_rule((2,0), 0, 0, 1)
         self.table.apply_stylesheet(self.stylesheet)
-        val = self.table.get_element(2,0)
+        val = self.table.get_cell(2,0)
         self.assert_(val == 22)
 
+
+
 class TableStyleTC(unittest.TestCase):
     """Test suite for TableSuite"""
     def setUp(self):
@@ -330,15 +410,6 @@ class DocbookTableWriterTC(testlib.TestCase):
         writer = TableWriter(self.stream, self.table, None)
         self.assertRaises(NotImplementedError, writer.write_table)
     
-def suite():
-    loader = unittest.TestLoader()
-    testsuite = loader.loadTestsFromModule(sys.modules[__name__])
-    return testsuite    
-    
-def Run():
-    testsuite = suite()
-    runner = unittest.TextTestRunner()
-    return runner.run(testsuite)
     
 if __name__ == '__main__':
-    Run()
+    unittest.main()
