@@ -1,7 +1,7 @@
 '''unit tests for ureports.text_writer
 '''
 
-__revision__ = "$Id: unittest_ureports_text.py,v 1.3 2005/04/26 16:51:13 syt Exp $"
+__revision__ = "$Id: unittest_ureports_text.py,v 1.4 2005/05/27 12:27:08 syt Exp $"
 
 import unittest
 from utils import WriterTC
@@ -51,8 +51,12 @@ Sub section description
     
     # Table tests #############################################################
     table_base = '''
-head1 head2 
-cell1 cell2 
++------+------+
+|head1 |head2 |
++------+------+
+|cell1 |cell2 |
++------+------+
+
 '''
     field_table = '''
 f1  : v1
@@ -60,12 +64,18 @@ f22 : v22
 f333: v333
 '''
     advanced_table = '''
-field          value 
-:::::::::::::::::::::
-f1             v1    
-f22            v22   
-f333           v333  
-`toi perdu ?`_ 
++---------------+------+
+|field          |value |
++===============+======+
+|f1             |v1    |
++---------------+------+
+|f22            |v22   |
++---------------+------+
+|f333           |v333  |
++---------------+------+
+|`toi perdu ?`_ |      |
++---------------+------+
+
 '''
 
 
