@@ -23,13 +23,15 @@ If no non-option arguments are present, prefixes used are 'test',
 
 """
 
-__revision__ = "$Id: testlib.py,v 1.24 2005/04/07 15:42:08 syt Exp $"
+__revision__ = "$Id: testlib.py,v 1.29 2005/08/08 10:00:14 adim Exp $"
 
 import sys
 import os
 import getopt
 import traceback
 import unittest
+import difflib
+from warnings import warn
 
 try:
     from test import test_support
@@ -39,7 +41,9 @@ except ImportError:
         def unload(self, test):
             pass
     test_support = TestSupport()
-    
+
+from logilab.common import class_renamed
+from logilab.common.compat import set
 from logilab.common.modutils import load_module_from_name
 
 __all__ = ['main', 'find_tests', 'run_test', 'spawn']
@@ -235,13 +239,11 @@ def _count(n, word):
 
 
 # test utils ##################################################################
-from xml.sax import make_parser, SAXParseException
 from cStringIO import StringIO
 
 class TestCase(unittest.TestCase):
     """unittest.TestCase with some additional methods"""
 
-
     def assertDictEquals(self, d1, d2):
         """compares two dicts
 
@@ -258,8 +260,17 @@ class TestCase(unittest.TestCase):
                 self.fail('missing %r key' % key)
         if d1:
             self.fail('d2 is missing %r' % d1)
-    
-    def assertListEquals(self, l1, l2):
+
+    def assertSetEqual(self, got, expected):
+        """compares two iterables and shows difference between both"""
+        got, expected = set(got), set(expected)
+        if got != expected:
+            missing = expected - got
+            unexpected = got - expected
+            self.fail('\tunexepected: %s\n\tmissing: %s' % (unexpected,
+                                                            missing))
+
+    def assertListEqual(self, l1, l2):
         """compares two lists
 
         If the two list differ, the first difference is shown in the error
@@ -277,13 +288,16 @@ class TestCase(unittest.TestCase):
                     l2.index(value), len(l2), value))
         if l1:
             self.fail('l2 is missing %r' % l1)
+    assertListEquals = assertListEqual
     
     def assertLinesEquals(self, l1, l2):
         """assert list of lines are equal"""
         self.assertListEquals(l1.splitlines(), l2.splitlines())
+    assertLineEqual = assertLinesEquals
 
     def assertXMLValid(self, stream):
         """asserts the XML stream is well-formed (no DTD conformance check)"""
+        from xml.sax import make_parser, SAXParseException
         parser = make_parser()
         try:
             parser.parse(stream)
@@ -295,8 +309,28 @@ class TestCase(unittest.TestCase):
         stream = StringIO(xml_string)
         self.assertXMLValid(stream)
 
-        
-        
+
+    def assertStreamEqual(self, stream1, stream2, junk=(' ', '\t')):
+        """compare two streams (using difflib and readlines())"""
+        # if stream2 is stream2, readlines() on stream1 will also read lines
+        # in stream2, so they'll appear different, although they're not
+        if stream1 is stream2:
+            return
+        # result is a generator
+        result = difflib.ndiff(stream1.readlines(), stream2.readlines(),
+                               charjunk=lambda x: x in junk)
+        read = []
+        for line in result:
+            read.append(line)
+            # lines that don't start with a ' ' are diff ones
+            if not line.startswith(' '):
+                self.fail(''.join(read + list(result)))
+            
+    def assertFileEqual(self, fname1, fname2, junk=(' ', '\t')):
+        """compares two files using difflib"""
+        self.assertStreamEqual(file(fname1), file(fname2), junk)
+
+
 import doctest
 
 class DocTest(unittest.TestCase):
@@ -348,18 +382,26 @@ class MockConfigParser:
             return 0
     
 
-class MockConnexion:
+class MockConnection:
     """fake DB-API 2.0 connexion AND cursor (i.e. cursor() return self)"""
     
     def __init__(self, results):
         self.received = []
+        self.states = []
         self.results = results
         
     def cursor(self):
         return self
-    def execute(self, query):
-        self.received.append(query)
+    def execute(self, query, args=None):
+        self.received.append( (query, args) )
     def fetchone(self):
         return self.results[0]
     def fetchall(self):
-        return self.result
+        return self.results
+    def commit(self):
+        self.states.append( ('commit', len(self.received)) )
+    def rollback(self):
+        self.states.append( ('rollback', len(self.received)) )
+
+MockConnexion = class_renamed('MockConnexion', MockConnection)
+
