@@ -15,7 +15,7 @@
 # 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 """Some text manipulation utility functions.
 
-:version:   $Revision: 1.23 $  
+:version:   $Revision: 1.25 $  
 :author:    Logilab
 :copyright: 2003-2005 LOGILAB S.A. (Paris, FRANCE)
 :contact:   http://www.logilab.fr/ -- mailto:python-projects@logilab.org
@@ -46,7 +46,7 @@ unquote, colorize_ansi
   ANSI terminal code reseting format defined by a previous ANSI escape sequence
 """
 
-__revision__ = "$Id: textutils.py,v 1.23 2005/02/08 17:23:02 syt Exp $"
+__revision__ = "$Id: textutils.py,v 1.25 2005/09/06 08:51:01 alf Exp $"
 __docformat__ = "restructuredtext en"
 
 import re
@@ -92,9 +92,9 @@ _NORM_SPACES_RGX = re.compile('\s+')
 
 def normalize_text(text, line_len=80, indent=''):
     """normalize a text to display it with a maximum line size and
-    optionaly arbitrary indentation. Line jumps are normalized but blank
-    lines are kept. The indentation string may be used top insert a
-    comment mark for instance.
+    optionally arbitrary indentation. Line jumps are normalized but blank
+    lines are kept. The indentation string may be used to insert a
+    comment (#) or a quoting (>) mark  for instance.
 
     :type text: str or unicode
     :param text: the input text to normalize
@@ -114,7 +114,8 @@ def normalize_text(text, line_len=80, indent=''):
     result = []
     for text in _BLANKLINES_RGX.split(text):
         result.append(normalize_paragraph(text, line_len, indent))
-    return ('%s%s%s' % (linesep, indent, linesep)).join(result)
+##     return ('%s%s%s' % (linesep, indent, linesep)).join(result)
+    return ('%s%s' % (linesep, linesep)).join(result)
 
 def normalize_paragraph(text, line_len=80, indent=''):
     """normalize a text to display it with a maximum line size and
@@ -153,7 +154,7 @@ def normalize_paragraph(text, line_len=80, indent=''):
                 pos = pos - len(indent)
                 while text[pos] != ' ':
                     pos += 1
-        lines.append(indent + text[:pos])
+        lines.append((indent + text[:pos]))
         text = text[pos+1:]
     return linesep.join(lines)
 
