@@ -21,11 +21,12 @@ formatted as text and html
 
 from __future__ import generators
 
-__revision__ = "$Id: __init__.py,v 1.6 2005/04/26 16:50:20 syt Exp $"
+__revision__ = "$Id: __init__.py,v 1.8 2005/07/02 13:22:30 syt Exp $"
 
 import sys
 from os import linesep
 from cStringIO import StringIO
+from StringIO import StringIO as UStringIO
 
 
 def get_nodes(node, klass):
@@ -58,20 +59,34 @@ def build_summary(layout, level=1):
         if not child.id:
             child.id = label.replace(' ', '-')
         node = Link('#'+child.id, label=label or child.id)
+        # FIXME: Three following lines produce not very compliant
+        # docbook: there are some useless <para><para>. They might be
+        # replaced by the three commented lines but this then produces
+        # a bug in html display...
         if level and [n for n in child.children if isinstance(n, Section)]:
             node = Paragraph([node, build_summary(child, level)])
         summary.append(node)
+#         summary.append(node)
+#         if level and [n for n in child.children if isinstance(n, Section)]:
+#             summary.append(build_summary(child, level))
     return summary
 
 
 class BaseWriter(object):
     """base class for ureport writers"""
     
-    def format(self, layout, stream=None):
+    def format(self, layout, stream=None, encoding=None):
         """format and write the given layout into the stream object
+
+        unicode policy: unicode strings may be found in the layout;
+        try to call stream.write with it, but give it back encoded using
+        the given encoding if it fails
         """
         if stream is None:
             stream = sys.stdout
+        if not encoding:
+            encoding = getattr(stream, 'encoding', 'UTF-8')
+        self.encoding = encoding or 'UTF-8'
         self.__compute_funcs = []
         self.out = stream
         self.begin_format(layout)
@@ -87,11 +102,14 @@ class BaseWriter(object):
 
     def writeln(self, string=''):
         """write a line in the output buffer"""
-        self.out.write(string + linesep)
+        self.write(string + linesep)
 
     def write(self, string):
         """write a string in the output buffer"""
-        self.out.write(string)
+        try:
+            self.out.write(string)
+        except UnicodeEncodeError:
+            self.out.write(string.encode(self.encoding))
 
     def begin_format(self, layout):
         """begin to format a layout"""
@@ -126,14 +144,20 @@ class BaseWriter(object):
         """
         # use cells !
         def write(data):
-            stream.write(data)
+            try:
+                stream.write(data)
+            except UnicodeEncodeError:
+                stream.write(data.encode(self.encoding))
         def writeln(data=''):
-            stream.write(data+linesep)
+            try:
+                stream.write(data+linesep)
+            except UnicodeEncodeError:
+                stream.write(data.encode(self.encoding)+linesep)
         self.write = write
         self.writeln = writeln
         self.__compute_funcs.append((write, writeln))
         for child in layout.children:
-            stream = StringIO()
+            stream = UStringIO()
             child.accept(self)
             yield stream.getvalue()
         self.__compute_funcs.pop()
