@@ -18,7 +18,7 @@
 A Universal report is a tree of layout and content objects
 """
 
-__revision__ = "$Id: nodes.py,v 1.5 2005/04/26 16:41:29 syt Exp $"
+__revision__ = "$Id: nodes.py,v 1.7 2005/07/07 07:09:05 syt Exp $"
 
 from logilab.common.tree import VNode
 
@@ -72,13 +72,13 @@ class Text(BaseComponent):
 
     attributes :
     * BaseComponent attributes
-    * data : the text value as an encoded string (REQUIRED)
+    * data : the text value as an encoded or unicode string
     """
     def __init__(self, data, **kwargs):
         super(Text, self).__init__(**kwargs)
-        if isinstance(data, unicode):
-            data = data.encode('ascii')
-        assert isinstance(data, str), data.__class__
+        #if isinstance(data, unicode):
+        #    data = data.encode('ascii')
+        assert isinstance(data, (str, unicode)), data.__class__
         self.data = data
 
 class VerbatimText(Text):
@@ -86,7 +86,7 @@ class VerbatimText(Text):
 
     attributes :
     * BaseComponent attributes
-    * data : the text value as an encoded string (REQUIRED)
+    * data : the text value as an encoded or unicode string
     """
         
 class Link(BaseComponent):
@@ -95,7 +95,7 @@ class Link(BaseComponent):
     attributes :
     * BaseComponent attributes
     * url : the link's target (REQUIRED)
-    * label : the link's label as an encoded string (use the url by default)
+    * label : the link's label as a string (use the url by default)
     """
     def __init__(self, url, label=None, **kwargs):
         super(Link, self).__init__(**kwargs)
