@@ -15,14 +15,14 @@
 # 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 """Text formatting drivers for ureports"""
 
-__revision__ = "$Id: text_writer.py,v 1.5 2005/04/26 16:48:34 syt Exp $"
+__revision__ = "$Id: text_writer.py,v 1.8 2005/05/27 12:27:09 syt Exp $"
 
 from os import linesep
 from operator import add
 
 from logilab.common.ureports import BaseWriter
 
-TITLE_UNDERLINES = ['', '=', '-', '`', '.']
+TITLE_UNDERLINES = ['', '=', '-', '`', '.', '~', '^']
 BULLETS = ['*', '-']
  
 class TextWriter(BaseWriter):
@@ -54,7 +54,7 @@ class TextWriter(BaseWriter):
         try:
             self.writeln(TITLE_UNDERLINES[self.section] * len(title))
         except IndexError:
-            pass
+            print "FIXME TITLE TOO DEEP. TURNING TITLE INTO TEXT"
         
     def visit_paragraph(self, layout):
         """enter a paragraph"""
@@ -86,16 +86,20 @@ class TextWriter(BaseWriter):
         format_strings = ' '.join(['%%-%ss'] * len(cols_width))
         format_strings = format_strings % tuple(cols_width)
         format_strings = format_strings.split(' ')
-        sep = ':' * reduce(add, cols_width)
+        linesep = '\n+' + '+'.join(['-'*w for w in cols_width]) + '+\n'
+        headsep = '\n+' + '+'.join(['='*w for w in cols_width]) + '+\n'
         # FIXME: layout.cheaders
+        self.write(linesep)
         for i in range(len(table_content)):
-            self.writeln()
+            self.write('|')
             line = table_content[i]
             for j in range(len(line)):
                 self.write(format_strings[j] % line[j])
+                self.write('|')
             if i == 0 and layout.rheaders:
-                self.writeln()
-                self.write(sep)
+                self.write(headsep)
+            else:
+                self.write(linesep)
  
     def field_table(self, layout, table_content, cols_width):
         """special case for field table"""
